import { store } from './redux/store';
import {
  asyncLogin,
  asyncLogout,
  asyncCreateUser,
} from './redux/authorization/operations';
import {
  asyncGetCategories,
  asyncGetCategoryById,
} from './redux/categories/operations';
import { asyncGetGoods, asyncGetGoodById } from './redux/goods/operations';
import { asyncGetOrders, asyncGetOrderById } from './redux/orders/operations';
import { getOrders, getOrder } from './redux/orders/selector';
import { getToken } from './redux/authorization/selector';
import { getCategories, getCategory } from './redux/categories/selector';
import { getGoods, getGood } from './redux/goods/selector/';
const { dispatch, subscribe, getState } = store;

const statusHtml = document.getElementById('statusTitle');
const formAuthHtml = document.getElementById('authForm');
const changeStatus = token => {
  if (token) {
    statusHtml.textContent = 'Status : logIn';
    statusHtml.style.color = 'green';
  } else {
    statusHtml.textContent = 'Status : Not logIn';
    statusHtml.style.color = 'red';
  }
};

subscribe(() => changeStatus(getToken(getState())));

const handleAuthForm = async function (e) {
  e.preventDefault();
  const id = e.target.id;
  const form = this.children;
  const login = form[0].value;
  const password = form[1].value;
  if (id === 'logout') {
    return dispatch(asyncLogout());
  } else if (login === '' || password === '') {
    return;
  } else if (id === 'login') {
    dispatch(asyncLogin(login, password));
  } else if (id === 'register') {
    dispatch(asyncCreateUser(login, password));
  }
};

formAuthHtml.addEventListener('click', handleAuthForm);

const renderListHtml = document.getElementById('renderList');

const handleRenderListHtml = function (e) {
  e.preventDefault();
  const id = e.target.id;
  const name = e.target.name;
  if (!id) return;
  if (name === 'category') {
    dispatch(asyncGetCategoryById(id));
    subscribe(() => renderList(getCategory(getState()), renderListHtml, ''));
  } else if (name === 'good') {
    dispatch(asyncGetGoodById(id));
    subscribe(() => renderList(getGood(getState()), renderListHtml, ''));
  } else if (name === 'order') {
    dispatch(asyncGetOrderById(id));
    subscribe(() => renderList(getOrder(getState()), renderListHtml, ''));
  }
};

renderListHtml.addEventListener('click', handleRenderListHtml);

const renderList = (data, root, requestName) => {
  root.innerHTML = '';
  const arr = data.map(({ _id, name, createdAt }) => {
    const li = document.createElement('li');
    const id = document.createElement('span');
    id.textContent = `id : ${_id}`;
    const nameHtml = document.createElement('span');
    nameHtml.textContent = `name : ${name ? name : 'missing'}`;
    const created = document.createElement('span');
    created.textContent = `createdAt : ${createdAt}`;
    const btn = document.createElement('button');
    btn.textContent = 'Detail By Id';
    btn.classList.add('btnDetail');
    btn.id = _id;
    btn.name = requestName;
    li.append(id, nameHtml, created, requestName ? btn : '');
    return li;
  });
  root.append(...arr);
};

const handleList = async function (e) {
  e.preventDefault();
  const id = e.target.id;
  if (id === 'categoriesBtn') {
    dispatch(asyncGetCategories());
    subscribe(() =>
      renderList(getCategories(getState()), renderListHtml, 'category'),
    );
  } else if (id === 'goodsBtn') {
    dispatch(asyncGetGoods());
    subscribe(() => renderList(getGoods(getState()), renderListHtml, 'good'));
    renderList(data, renderListHtml);
  } else if (id === 'ordersBtn') {
    dispatch(asyncGetOrders());
    subscribe(() => renderList(getOrders(getState()), renderListHtml, 'order'));
  }
};

const btnWrapper = document.getElementById('btnWrapper');
btnWrapper.addEventListener('click', handleList);
